/**
 *      
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The Original Code is Copyright (C) 2006 by Bernd Breitenbach.
 * All rights reserved.
 *
 * The Original Code is: all of this file.
 *
 */


/**
 * \file labystruct.c
 * \brief 
 * \author Bernd Breitenbach
 * \remarks
 *      bb - Sep 1, 2006: Created.
 */

#include "mazelib.h"

/**********************************************************************/

#define _2A    (2.*_A)
#define _3A    (3.*_A)
#define _4A    (4.*_A)
#define _A_2   (0.5*_A)
#define _A_3   (_A/3.)
#define _3A_2  (1.5*_A)
#define _2A_3  (2.*_A/3.)

#define _2B    (2.*_B)
#define _3B    (3.*_B)
#define _4B    (4.*_B)
#define _B_2   (0.5*_B)
#define _B_3   (_B/3.)
#define _3B_2  (1.5*_B)
#define _2B_3  (2.*_B/3.)

#define _AB    (_A+_B)

#define _N     (9E9)
/**********************************************************************/
/*                 orthogonal structure                               */
/**********************************************************************/


#define _A (1.)

cell_structure_t ortho_struct=
{
  .name="Orthogonal",
  .rows   =1,
  .columns=1,
  .width  =_A,
  .height =_A,
  .xminadd =-_A_2,
  .xmaxadd =_A_2,
  .yminadd =-_A_2,
  .ymaxadd =_A_2,
  .center_points=(coords_u[]) {
    {.cartesic={0.,0.}}
  },
  .cell_types=(cell_type_t[]) {
    TYPE_A
  },
  .n_neighbours=(int[]){4},
  .cell_coords={
    (border_t[]){ /* TYPE_A */
      {{.cartesic={ _A_2  ,  _A_2 }},  0,  1}, /* N  */
      {{.cartesic={-_A_2  ,  _A_2 }}, -1,  0}, /* W  */
      {{.cartesic={-_A_2  , -_A_2 }},  0, -1}, /* S  */
      {{.cartesic={ _A_2  , -_A_2 }},  1,  0}, /* E  */
      {{.cartesic={ _A_2  ,  _A_2 }},  0,  1} 
    } 
  }
};

#undef _A


/**********************************************************************/
/*                 triangle structure                                 */
/**********************************************************************/

/* sqrt(3)/2 */
#define TRIA_A (0.866025403784438596588302061718)
/* 1/2 */
#define TRIA_B (0.5)

#define _A (1.)
#define _B (TRIA_B/TRIA_A)

cell_structure_t tria_struct=
{
  .name="Triagonal",
  .rows   =2,
  .columns=2,
  .width  =_2B,
  .height =_2A,
  .xminadd =0.,
  .xmaxadd =_B,
  .yminadd =-_A_2,
  .ymaxadd =_A_2,
  .center_points=(coords_u[]) {
    {.cartesic={0.,0.}},{.cartesic={_B,0.}},
    {.cartesic={0.,_A}},{.cartesic={_B,_A}},
  },
  .cell_types=(cell_type_t[]) {
    TYPE_A,TYPE_B,
    TYPE_B,TYPE_A,
  },
  .n_neighbours=(int[]){3,3},
  .cell_coords={
    (border_t[]){ /* TYPE_A */
      {{.cartesic={   0.  ,  _A_2 }}, -1,  0}, /* NW */
      {{.cartesic={  -_B  , -_A_2 }},  0, -1}, /* S  */
      {{.cartesic={   _B  , -_A_2 }},  1,  0}, /* NE */
      {{.cartesic={   0.  ,  _A_2 }}, -1,  0}, 
    },(border_t[]){ /* TYPE_B */ 
      {{.cartesic={   _B  ,  _A_2 }},  0,  1}, /* N  */
      {{.cartesic={  -_B  ,  _A_2 }}, -1,  0}, /* SW */
      {{.cartesic={   0.  , -_A_2 }},  1,  0}, /* SE */
      {{.cartesic={   _B  ,  _A_2 }},  0,  1}, 
    }  
  }
};

#undef _A
#undef _B


/**********************************************************************/
/*                 hexagon structure                                  */
/**********************************************************************/

/* sqrt(3) */
#define HEXA_A (1.73205080756887719317660412344)
/* 3/2 */
#define HEXA_B (1.5)

#define _A (1.)
#define _B (HEXA_B/HEXA_A)

cell_structure_t hexa_struct=
{
  .name="Hexagonal",
  .rows   =2,
  .columns=2,
  .width  =_2B,
  .height =_2A,
  .xminadd =-_B_3,
  .xmaxadd =_2B_3,
  .yminadd =-_A_2,
  .ymaxadd =0.,
  .center_points=(coords_u[]) {
    {.cartesic={0.,0.}},{.cartesic={_B,_A_2}},
    {.cartesic={0.,_A}},{.cartesic={_B,_3A_2}}
  },
  .cell_types=(cell_type_t[]) {
    TYPE_A,TYPE_B,
    TYPE_A,TYPE_B,
  },
  .n_neighbours=(int[]){6,6},
  .cell_coords={/* TYPE_A */
    (border_t[]){ 
      {{.cartesic={ _B_3  ,  _A_2 }}, 0, 1}, /* N  */
      {{.cartesic={-_B_3  ,  _A_2 }},-1, 0}, /* NW */
      {{.cartesic={-_2B_3 ,    0. }},-1,-1}, /* SW */
      {{.cartesic={-_B_3  , -_A_2 }}, 0,-1}, /* S  */
      {{.cartesic={ _B_3  , -_A_2 }}, 1,-1}, /* SE */
      {{.cartesic={ _2B_3 ,    0. }}, 1, 0}, /* NE */
      {{.cartesic={ _B_3  ,  _A_2 }}, 0, 1} 
    },(border_t[]){ /* TYPE_B */
      {{.cartesic={ _B_3  ,  _A_2 }}, 0, 1}, /* N  */
      {{.cartesic={-_B_3  ,  _A_2 }},-1, 1}, /* NW */
      {{.cartesic={-_2B_3 ,    0. }},-1, 0}, /* SW */
      {{.cartesic={-_B_3  , -_A_2 }}, 0,-1}, /* S  */
      {{.cartesic={ _B_3  , -_A_2 }}, 1, 0}, /* SE */
      {{.cartesic={ _2B_3 ,    0. }}, 1, 1}, /* NE */
      {{.cartesic={ _B_3  ,  _A_2 }}, 0, 1} 
    } 
  }
};

#undef _A
#undef _B



/**********************************************************************/
/*                 hexagonal-square structure                         */
/**********************************************************************/


#define _A (1.)

cell_structure_t hexa_square_struct=
{
  .name="Hexagon-Square",
  .rows   =2,
  .columns=2,
  .width  =_2A,
  .height =_2A,
  .xminadd =0.,
  .xmaxadd =_A,
  .yminadd =-_A_2,
  .ymaxadd =_A_2,
  .center_points=(coords_u[]) {
    {.cartesic={_N, _N}},{.cartesic={_A, 0.}},
    {.cartesic={0., _A}},{.cartesic={_A,_A}},
  },
  .cell_types=(cell_type_t[]) {
    TYPE_NONE,TYPE_C,
    TYPE_A   ,TYPE_B,
  },
  .n_neighbours=(int[]){6,4,6},
  .cell_coords={
    (border_t[]){ /* TYPE_A  (vertical hexagon) */
      {{.cartesic={   0.  ,  _A   }}, -1,  1}, /* NW */
      {{.cartesic={ -_A_2 ,  _A_2 }}, -1,  0}, /* W  */
      {{.cartesic={ -_A_2 , -_A_2 }}, -1, -1}, /* SW */
      {{.cartesic={   0.  , -_A   }},  1, -1}, /* SE */
      {{.cartesic={  _A_2 , -_A_2 }},  1,  0}, /* E  */
      {{.cartesic={  _A_2 ,  _A_2 }},  1,  1}, /* NE */
      {{.cartesic={   0.  ,  _A   }}, -1,  1} 
    },(border_t[]){ /* TYPE_B (square) */ 
      {{.cartesic={ _A_2  ,  _A_2 }},  0,  1}, /* N  */
      {{.cartesic={-_A_2  ,  _A_2 }}, -1,  0}, /* W  */
      {{.cartesic={-_A_2  , -_A_2 }},  0, -1}, /* S  */
      {{.cartesic={ _A_2  , -_A_2 }},  1,  0}, /* E  */
      {{.cartesic={ _A_2  ,  _A_2 }},  0,  1} 
    }, (border_t[]){ /* TYPE_C (horizontal hexagon) */
      {{.cartesic={ _A_2  ,  _A_2 }},  0,  1}, /* N  */
      {{.cartesic={-_A_2  ,  _A_2 }}, -1,  1}, /* NW */
      {{.cartesic={-_A   ,    0.  }}, -1, -1}, /* SW */
      {{.cartesic={-_A_2  , -_A_2 }},  0, -1}, /* S  */
      {{.cartesic={ _A_2  , -_A_2 }},  1, -1}, /* SE */
      {{.cartesic={ _A    ,   0.  }},  1,  1}, /* NE */
      {{.cartesic={ _A_2  ,  _A_2 }},  0,  1}
    }  
  }
};

#undef _A

/**********************************************************************/
/*                 cairo structure                                    */
/**********************************************************************/

/*   1/4*(sqrt(7)-1) */
#define CAIRO_A (0.411437827766147679042774143454) 
/*   1/4*(sqrt(7)+1) */
#define CAIRO_B (0.911437827766147679042774143454)


#define _A (CAIRO_A/(CAIRO_A+CAIRO_B))
#define _B (CAIRO_B/(CAIRO_A+CAIRO_B))

cell_structure_t cairo_struct=
{
  .name="Cairo",
  .rows   =3,
  .columns=3,
  .width  =_4B,
  .height =_4B,
  .xminadd =0.,
  .xmaxadd =0.,
  .yminadd =0.,
  .ymaxadd =0.,
  .center_points=(coords_u[]) {
    {.cartesic={0., 0.}},{.cartesic={_2B, 0.}},{.cartesic={_3B,-_B}},
    {.cartesic={_B, _B}},{.cartesic={_N , _N}},{.cartesic={_3B, _B}},
    {.cartesic={_B,_3B}},{.cartesic={_2B,_2B}},{.cartesic={_4B,_2B}}
  },
  .cell_types=(cell_type_t[]) {
    TYPE_A,TYPE_B   ,TYPE_C,
    TYPE_G,TYPE_NONE,TYPE_D,
    TYPE_H,TYPE_E   ,TYPE_F,
  },
  .n_neighbours=(int[]){5,5,5,5,5,5,5,5},
  .cell_coords={    /* TYPE_A */
    (border_t[]){ 
      {{.cartesic={   0.  ,  _B   }},    -1, 1}, /* NW */
      {{.cartesic={  -_A  ,  0.   }},    -1, 0}, /* SW */
      {{.cartesic={   0.  , -_B   }},     0,-1}, /* SE */
      {{.cartesic={   _B  , _A-_B }},     1, 0}, /* E  */
      {{.cartesic={   _B  , _B-_A }},     0, 1}, /* NE */
      {{.cartesic={   0.  ,  _B   }},    -1, 1} 
    },(border_t[]){ /* TYPE_B */
      {{.cartesic={   0.  ,  _B   }},    -1, 1}, /* NW */
      {{.cartesic={  -_B  , _B-_A }},    -1, 0}, /* W  */
      {{.cartesic={  -_B  , _A-_B }},    -1,-1}, /* SW */
      {{.cartesic={   0.  ,  -_B  }},     1, 0}, /* SE */
      {{.cartesic={   _A  ,  0.   }},     1, 1}, /* NE */
      {{.cartesic={   0.  ,  _B   }},    -1, 1}, /* NW */
    },(border_t[]){ /* TYPE_C */ 
      {{.cartesic={ _B-_A ,  _B   }},     0, 1}, /* N  */
      {{.cartesic={ _A-_B ,  _B   }},    -1, 0}, /* NW */
      {{.cartesic={   -_B ,  0.   }},    -1,-1}, /* SW */
      {{.cartesic={    0. , -_A   }},     0,-1}, /* SE */
      {{.cartesic={    _B ,  0.   }},     1, 0}, /* NE */
      {{.cartesic={ _B-_A ,  _B   }},     0, 1}, 
    },(border_t[]){ /* TYPE_D */
      {{.cartesic={    0. ,  _A   }},    -1, 1}, /* NW */
      {{.cartesic={   -_B ,  0.   }},    -1,-1}, /* SW */
      {{.cartesic={ _A-_B , -_B   }},     0,-1}, /* S  */
      {{.cartesic={ _B-_A , -_B   }},     1,-1}, /* SE */
      {{.cartesic={    _B ,  0.   }},     0, 1}, /* NE */
      {{.cartesic={    0. ,  _A   }},    -1, 1}, 
    },(border_t[]){ /* TYPE_E */
      {{.cartesic={   0.  ,  _B   }},    -1, 0}, /* NW */
      {{.cartesic={  -_A  ,  0.   }},    -1,-1}, /* SW */
      {{.cartesic={   0.  , -_B   }},     1,-1}, /* SE */
      {{.cartesic={   _B  , _A-_B }},     1, 0}, /* E  */
      {{.cartesic={   _B  , _B-_A }},     1, 1}, /* NE */
      {{.cartesic={   0.  ,  _B   }},    -1, 0}, 
    },(border_t[]){ /* TYPE_F */
      {{.cartesic={   0.  ,  _B   }},     0, 1}, /* NW */
      {{.cartesic={  -_B  , _B-_A }},    -1, 0}, /* W  */
      {{.cartesic={  -_B  , _A-_B }},     0,-1}, /* SW */
      {{.cartesic={   0.  ,  -_B  }},     1,-1}, /* SE */
      {{.cartesic={   _A  ,  0.   }},     1, 0}, /* NE */
      {{.cartesic={   0.  ,  _B   }},    -1, 1}, 
    },(border_t[]){ /* TYPE_G */
      {{.cartesic={ _B-_A ,  _B   }},     0, 1}, /* N  */
      {{.cartesic={ _A-_B ,  _B   }},    -1, 1}, /* NW */
      {{.cartesic={   -_B ,  0.   }},     0,-1}, /* SW */
      {{.cartesic={    0. , -_A   }},     1,-1}, /* SE */
      {{.cartesic={    _B ,  0.   }},     1, 1}, /* NE */
      {{.cartesic={ _B-_A ,  _B   }},     0, 1}, 
    },(border_t[]){ /* TYPE_H */
      {{.cartesic={    0. ,  _A   }},     0, 1}, /* NW */
      {{.cartesic={   -_B ,  0.   }},    -1, 0}, /* SW */
      {{.cartesic={ _A-_B , -_B   }},     0,-1}, /* S  */
      {{.cartesic={ _B-_A , -_B   }},     1, 0}, /* SE */
      {{.cartesic={    _B ,  0.   }},     1, 1}, /* NE */
      {{.cartesic={    0. ,  _A   }},     0, 1}, 
    } 
  }
};

#undef _A
#undef _B



/**********************************************************************/
/*                 octagonal-square structure                         */
/**********************************************************************/

/* 1/2 */
#define OS_A (0.5)

/* sqrt(1/2)+1/2 */
#define OS_B (1.20710678118654746171500846685)

#define _A (OS_A/(OS_A+OS_B))
#define _B (OS_B/(OS_A+OS_B))

cell_structure_t octa_square_struct=
{
  .name="Octagon-Square",
  .rows   =2,
  .columns=2,
  .width  =2.*_AB,
  .height =2.*_AB,
  .xminadd =-_A,
  .xmaxadd =_B,
  .yminadd =-_A,
  .ymaxadd =_B,
  .center_points=(coords_u[]) {
    {.cartesic={0., 0.}},{.cartesic={_AB, 0.}},
    {.cartesic={0.,_AB}},{.cartesic={_AB,_AB}},
  },
  .cell_types=(cell_type_t[]) {
    TYPE_A,TYPE_B,
    TYPE_B,TYPE_A,
  },
  .n_neighbours=(int[]){4,8},
  .cell_coords={
    (border_t[]){ /* TYPE_A (square) */ 
      {{.cartesic={ _A  ,  _A }},  0,  1}, /* N  */
      {{.cartesic={-_A  ,  _A }}, -1,  0}, /* W  */
      {{.cartesic={-_A  , -_A }},  0, -1}, /* S  */
      {{.cartesic={ _A  , -_A }},  1,  0}, /* E  */
      {{.cartesic={ _A  ,  _A }},  0,  1} 
    },(border_t[]){ /* TYPE_B (octagon) */
      {{.cartesic={  _A ,  _B }},  0,  1}, /* N  */
      {{.cartesic={ -_A ,  _B }}, -1,  1}, /* NW */
      {{.cartesic={ -_B ,  _A }}, -1,  0}, /* W  */
      {{.cartesic={ -_B , -_A }}, -1, -1}, /* SW */
      {{.cartesic={ -_A , -_B }},  0, -1}, /* S  */
      {{.cartesic={  _A , -_B }},  1, -1}, /* SE */
      {{.cartesic={  _B , -_A }},  1,  0}, /* E  */
      {{.cartesic={  _B ,  _A }},  1,  1}, /* NE */
      {{.cartesic={  _A ,  _B }},  0,  1}
    }  
  }
};

#undef _A
#undef _B

/**********************************************************************/
/*                 rhombic structure                                  */
/**********************************************************************/


#define _A (1.)

cell_structure_t rhombic_struct=
{
  .name="Rhombic",
  .rows   =2,
  .columns=2,
  .width  =_2A,
  .height =_2A,
  .xminadd =0.,
  .xmaxadd =_A,
  .yminadd =0.,
  .ymaxadd =_A,
  .center_points=(coords_u[]) {
    {.cartesic={0.,0.}},{.cartesic={_N,_N}},
    {.cartesic={_N,_N}},{.cartesic={_A,_A}}
  },
  .cell_types=(cell_type_t[]) {
    TYPE_A,TYPE_NONE,
    TYPE_NONE,TYPE_A,
  },
  .n_neighbours=(int[]){4},
  .cell_coords={/* TYPE_A */
    (border_t[]){ 
      {{.cartesic={  0. ,  _A }}, -1, 1}, /* NW  */
      {{.cartesic={-_A  ,  0. }}, -1,-1}, /* SW  */
      {{.cartesic={  0. , -_A }},  1,-1}, /* SE  */
      {{.cartesic={ _A  ,  0. }},  1, 1}, /* NE  */
      {{.cartesic={  0. ,  _A }}, -1, 1}
    } 
  }
};

#undef _A


/**********************************************************************/
/*                 hexagonal-rhombic structure                        */
/**********************************************************************/


/* 3/4 */
#define HR_A (0.75)
/* 1/4*sqrt(3) */
#define HR_B  (0.4330127018922192982941510308592114597559)

#define _A (1.)
#define _B (HR_B/HR_A)
#define _C (0.5/HR_A)

cell_structure_t hexa_rhombic_struct=
{
  .name="Hexagonal-Rhombic",
  .rows   =4,
  .columns=4,
  .width  =_4B,
  .height =_4A,
  .xminadd =+_B,
  .xmaxadd =+_2B,
  .yminadd =-_A,
  .ymaxadd =0.,
  .center_points=(coords_u[]) {
    {.cartesic={0.,0. }},{.cartesic={_N,_N }},{.cartesic={_2B,0. }},{.cartesic={ _N,_N}},
    {.cartesic={_N,_N }},{.cartesic={_N,_N }},{.cartesic={_N,_N  }},{.cartesic={_3B,_A}},
    {.cartesic={0.,_2A}},{.cartesic={_N,_N }},{.cartesic={_2B,_2A}},{.cartesic={_N ,_N}},
    {.cartesic={_N,_N }},{.cartesic={_B,_3A}},{.cartesic={_N,_N  }},{.cartesic={_N ,_N}},
  },
  .cell_types=(cell_type_t[]) {
    TYPE_A   ,TYPE_NONE,TYPE_B   ,TYPE_NONE,
    TYPE_NONE,TYPE_NONE,TYPE_NONE,TYPE_C,
    TYPE_B   ,TYPE_NONE,TYPE_A   ,TYPE_NONE,
    TYPE_NONE,TYPE_C   ,TYPE_NONE,TYPE_NONE,
  },
  .n_neighbours=(int[]){4,4,4},
  .cell_coords={/* TYPE_A */
    (border_t[]){ 
      {{.cartesic={ _B  ,  _A    }}, -1, 1}, /* N  */
      {{.cartesic={-_B  ,  _A-_C }}, -2, 0}, /* W  */
      {{.cartesic={-_B  , -_A    }},  1,-1}, /* S  */
      {{.cartesic={ _B  , -_A+_C }},  2, 0}, /* E  */
      {{.cartesic={ _B  ,  _A    }}, -1, 1}
    } , (border_t[]){ /* TYPE_B */
      {{.cartesic={ _B  ,  _A-_C }},  1, 1}, /* N  */
      {{.cartesic={-_B  ,  _A    }}, -2, 0}, /* W  */
      {{.cartesic={-_B  , -_A+_C }}, -1,-1}, /* S  */
      {{.cartesic={ _B  , -_A    }},  2, 0}, /* E  */
      {{.cartesic={ _B  ,  _A-_C }},  1, 1}
    } , (border_t[]){ /* TYPE_C */
      {{.cartesic={  0. ,  _C }}, -1, 1}, /* NW */
      {{.cartesic={-_2B ,   0 }}, -1,-1}, /* SW */
      {{.cartesic={  0. , -_C }},  1,-1}, /* SE */
      {{.cartesic={ _2B ,   0.}},  1, 1}, /* NE */
      {{.cartesic={  0. ,  _C }}, -1, 1}
    } 
  }
};

#undef _A
#undef _B



/**********************************************************************/
/*                 triangle-square structure                          */
/**********************************************************************/

/* cos(15 Deg) */
#define _A (0.9659258262890683122137147620378527790308)
/* sin(15 Deg) */
#define _B (0.2588190451025207394764038326684385538101)

/* sqrt(3)/2 */
#define _TH (0.8660254037844385965883020617184229195118)
/* 1/3*sqrt(3) */
#define _TC (0.5773502691896257310588680411456152796745)
/* 1/2*sqrt(2) */
#define _SC (0.7071067811865475727373109293694142252207)

cell_structure_t triangle_square_struct=
{
  .name="Triangle-Square",
  .rows   =2,
  .columns=3,
  .width  =_2A,
  .height =_2A,
#if 0
  .xminadd =-1,
  .xmaxadd =1,
  .yminadd =-1,
  .ymaxadd =1,
#else
  .xminadd =-_B*_TC,
  .xmaxadd =_B+_TC*_B,
  .yminadd =-_B*_TC,
  .ymaxadd =_B+_TC*_B,
#endif
  .center_points=(coords_u[]) {
    {.cartesic={_TC*_B, _TC*_A           }},{.cartesic={_TC*_A, _TC*_B           }},{.cartesic={(_TH+0.5)*_A,(_TH+0.5)*_B}},
    {.cartesic={(_TH+0.5)*_B,(_TH+0.5)*_A}},{.cartesic={(_TC+1.)*_SC,(_TC+1.)*_SC}},{.cartesic={(2.*_TC+1.)*_SC,(2.*_TC+1.)*_SC}}
  },
  .cell_types=(cell_type_t[]) {
    TYPE_A,TYPE_B,TYPE_C,
    TYPE_D,TYPE_E,TYPE_F
  },
  .n_neighbours=(int[]){3,3,4,4,3,3},
  .cell_coords={/* TYPE_A */
    (border_t[]){ 
      {{.cartesic={ _A-_B-_TC*_B,_A-_B-_TC*_A}},  0, 1}, /* NW  */
      {{.cartesic={-_B-_TC*_B   ,_A-_TC*_A   }}, -1, 0}, /* SW  */
      {{.cartesic={ -_TC*_B     ,-_TC*_A     }},  1, 0}, /* SE  */
      {{.cartesic={ _A-_B-_TC*_B,_A-_B-_TC*_A}},  0, 1}
    },(border_t[]){ /* TYPE_B */
      {{.cartesic={ _A-_B-_TC*_A,_A-_B-_TC*_B }}, -1, 0}, /* NW  */
      {{.cartesic={ -_TC*_A     ,-_TC*_B      }}, -1,-1}, /* SE  */
      {{.cartesic={ _A-_TC*_A   ,-_B-_TC*_B   }},  1, 0}, /* SW  */
      {{.cartesic={ _A-_B-_TC*_A,_A-_B-_TC*_B }}, -1, 0}
    },(border_t[]){ /* TYPE_C */
      {{.cartesic={ _2A-_B-(_TH+0.5)*_A,_A-(_TH+0.5)*_B   }}, -1, 1}, /* NW  */
      {{.cartesic={ _A-_B-(_TH+0.5)*_A ,_A-_B-(_TH+0.5)*_B}}, -1, 0}, /* NW  */
      {{.cartesic={ _A-(_TH+0.5)*_A    , -_B-(_TH+0.5)*_B }},  0,-1}, /* SE  */
      {{.cartesic={ _2A-(_TH+0.5)*_A   , -(_TH+0.5)*_B    }},  1, 0}, /* SW  */
      {{.cartesic={ _2A-_B-(_TH+0.5)*_A,_A-(_TH+0.5)*_B   }}, -1, 1}
    },(border_t[]){ /* TYPE_D */
      {{.cartesic={ _A-(_TH+0.5)*_B    ,_2A-_B-(_TH+0.5)*_A }},  1, 1}, /* NW  */
      {{.cartesic={ -(_TH+0.5)*_B      ,_2A-(_TH+0.5)*_A    }}, -1, 0}, /* NW  */
      {{.cartesic={-_B-(_TH+0.5)*_B    , _A-(_TH+0.5)*_A    }},  0,-1}, /* SE  */
      {{.cartesic={ _A-_B-(_TH+0.5)*_B ,_A-_B-(_TH+0.5)*_A  }},  1, 0}, /* NW  */
      {{.cartesic={ _A-(_TH+0.5)*_B    ,_2A-_B-(_TH+0.5)*_A }},  1, 1},
    },(border_t[]){ /* TYPE_E */
      {{.cartesic={ _A-(_TC+1.)*_SC    ,_2A-_B-(_TC+1.)*_SC}}, -1, 0}, /* NW  */
      {{.cartesic={ _A-_B-(_TC+1.)*_SC ,_A-_B-(_TC+1.)*_SC }},  1,-1}, /* NW  */
      {{.cartesic={ _2A-_B-(_TC+1.)*_SC,_A-(_TC+1.)*_SC    }},  1, 0}, /* SW  */
      {{.cartesic={ _A-(_TC+1.)*_SC    ,_2A-_B-(_TC+1.)*_SC}}, -1, 0}
    },(border_t[]){ /* TYPE_F */
      {{.cartesic={ _2A-(2.*_TC+1.)*_SC ,_2A-(2.*_TC+1.)*_SC     }},  0, 1}, /* NW  */
      {{.cartesic={ _A-(2.*_TC+1.)*_SC    ,_2A-_B-(2.*_TC+1.)*_SC}}, -1, 0}, /* NW  */
      {{.cartesic={ _2A-_B-(2.*_TC+1.)*_SC,_A-(2.*_TC+1.)*_SC    }},  1, 0}, /* SW  */
      {{.cartesic={ _2A-(2.*_TC+1.)*_SC   ,_2A-(2.*_TC+1.)*_SC   }},  0, 1}
    } 
  }
};

#undef _A
#undef _B
#undef _TH
#undef _TC
#undef _SC

/**********************************************************************/
